#!/usr/bin/python3

import asyncio
import json
import os
import subprocess
import sys
from itertools import islice
import datetime

from astra_event_viewer.reading_file import Logs


async def wait_updating_file(file_size_before_updating, count, args, way):
    tracking = True
    try:
        while tracking:
            filesize = os.path.getsize('/parsec/log/astra/events')
            if filesize > file_size_before_updating:
                file_size_before_updating = filesize
                await reading_lines(count, count=count, args=args, way=way)
            await asyncio.sleep(1)
    except KeyboardInterrupt:
        sys.exit()


async def logs_to_dataframe(line, way, source, priority, date, detailed, format, raw, nocolor, from_date, until_date,
                            time_format, count):
    try:
        one_log = list([])
        one_log.append(str(line))
        df_logs = Logs(one_log, way, source, priority, date, detailed, format, raw, nocolor, from_date, until_date,
                       time_format, count)
        df_logs.parse_file()
        await asyncio.sleep(0)
    except Exception as e:
        pass


async def reading_lines(place_of_reading, count, args, way):
    try:
        with open(way, 'r') as file1:
            for line in islice(file1, place_of_reading, None):
                count += 1
                await logs_to_dataframe(line, way, args.s, args.p, args.d, args.detailed, args.format, args.raw,
                                        args.nocolor, args.since, args.until, args.iso_date, args.count)
            filesize = os.path.getsize('/parsec/log/astra/events')
            if args.follow:
                await wait_updating_file(filesize, count, args, way)
    except Exception as e:
        print('No such directory')
        sys.exit(0)


def start_reading(args):
    way = args.w
    ioloop = asyncio.get_event_loop()
    if args.count:
        args.d = None
        lines_quantity = int(
            subprocess.run(['wc', '-l', way], stdout=subprocess.PIPE).stdout.decode().split(' ')[0])
        if lines_quantity < int(args.count):
            print('Set smaller value')
            sys.exit(1)
        if int(args.count) >= 1000:
            tasks = [ioloop.create_task(
                reading_lines(lines_quantity - int(args.count), lines_quantity - int(args.count), args, way))]
        else:
            tasks = [ioloop.create_task(get_last_n_events(args.count, args))]
    elif args.all or args.since or args.until:
        tasks = [ioloop.create_task(reading_lines(0, 0, args, way))]
    elif args.d != '0':
        needed_day = datetime.datetime.now() - datetime.timedelta(days=int(args.d))
        try:
            needed_file_line = int(subprocess.run(['grep', '-n', str(needed_day.date()), way, '-a', '-m', '1'],
                                                  stdout=subprocess.PIPE).stdout.decode().split(':')[0])
        except:
            print('Enter a smaller value')
            sys.exit(1)

        tasks = [ioloop.create_task(reading_lines(needed_file_line, needed_file_line, args, way))]
    else:
        tasks = [ioloop.create_task(get_last_n_events(1, args))]
    wait_tasks = asyncio.wait(tasks)
    ioloop.run_until_complete(wait_tasks)
    ioloop.close()
    sys.exit(0)


async def get_last_n_events(n, args):
    way = args.w
    n = int(n)
    try:
        with open(way, 'rb') as f:
            f.seek(-2, 2)
            buffer = bytearray()
            lines_count = 0
            while lines_count < n + 1:
                c = f.read(1)
                try:
                    buffer.insert(0, c[0])
                except KeyboardInterrupt:
                    sys.exit(1)
                except:
                    continue
                f.seek(-2, os.SEEK_CUR)
                if c == b'\n':
                    if args.d:
                        try:
                            today_date = datetime.datetime.fromisoformat(
                                json.loads(buffer.decode().split('\n')[1])['ISODATE']).date()
                        except:
                            try:
                                today_date = datetime.datetime.fromisoformat(
                                    datetime.datetime.strptime(
                                        find_key('time', json.loads(buffer.decode().split('\n')[1])),
                                        '%d.%m.%Y %H:%M:%S').isoformat()).date()
                            except:
                                try:
                                    today_date = datetime.datetime.fromisoformat(
                                        find_key('time', json.loads(buffer.decode().split('\n')[1]))).date()
                                except:
                                    try:
                                        today_date = datetime.datetime.fromisoformat(
                                            datetime.datetime.fromtimestamp(
                                                int(find_key('a', json.loads(buffer.decode().split('\n')[1])),
                                                    )).isoformat()).date()
                                    except:
                                        try:
                                            today_date = datetime.datetime.fromisoformat(
                                                datetime.datetime.strptime(
                                                    find_key('DATE', json.loads(buffer.decode().split('\n')[1])),
                                                    '%b %d %H:%M:%S').isoformat()).replace(
                                                year=datetime.datetime.now().year).date()
                                        except:
                                            pass
                        if args.d:
                            if today_date >= datetime.datetime.now().date() - datetime.timedelta(days=int(args.d)):
                                n += 1
                                if n >= 1000:
                                    await reading_lines(0, 0, args, way)
                                    sys.exit()
                            else:
                                n -= 1
                    lines_count += 1
            for line in buffer.decode().split('\n')[-n:]:
                await new_option_of_reading(line, args)
            file_size = int(subprocess.run(['wc', '-l', way], stdout=subprocess.PIPE).stdout.split()[0])
            if args.follow:
                await wait_updating_file(file_size, file_size, args, way)
    except Exception as e:
        print(e)


async def new_option_of_reading(line, args):
    args.d = False
    await logs_to_dataframe(line, args.w, args.s, args.p, args.d, args.detailed, args.format, args.raw,
                            args.nocolor, args.since, args.until, args.iso_date, args.count)


def find_key(key, dictionary):
    for k, v in dictionary.items():
        if k == key:
            return v
        elif isinstance(v, dict):
            result = find_key(key, v)
            if result is not None:
                return result
