#!/usr/bin/python3

import argparse
import gettext
import locale
import sys
from argparse import RawTextHelpFormatter

from astra_event_viewer import async_reading

gettext.bindtextdomain('translate_logs',
                       '/usr/share/locale')
gettext.textdomain('translate_logs')
_ = gettext.gettext

locale_info = locale.getdefaultlocale()[0]


def get_args():
    description = _(
        """A console utility for logging system events\nExample:\n astra-event-viewer (default only for today logs)\n astra-event-viewer -d 2 (logs for last 2 days)\n astra-event-viewer -w /path_to_file (default is /parsec/log/astra/events)\n astra-event-viewer -p alert notice (logs where priority is alert or notice)\n astra-event-viewer -s network (logs where source is network)\n astra-event-viewer --detailed (detailed format of output)\n astra-event-viewer --format=DATE: DATE\n to see all keys for format argument use --raw""")
    parser = argparse.ArgumentParser(description=description, formatter_class=RawTextHelpFormatter, add_help=True)

    parser.add_argument('--follow', '-l', action="store_true", help=_("check for updating logs' file"))
    parser.add_argument('-w', default='/parsec/log/astra/events', help=_("set path to the file of logs"))
    parser.add_argument('-p', nargs='+', help=_("priority filter"))
    parser.add_argument('-s', nargs='+', help=_("source filter"))
    parser.add_argument('-d', help=_("show logs for few last days"))
    parser.add_argument('--detailed', '-m', action="store_true", help=_("detailed information"))
    parser.add_argument('-t', '--today', action="store_true", help=_("show logs for today"))
    parser.add_argument('--format', '-f', help=_("set the output format"))
    parser.add_argument('--raw', '-r', action="store_true", help=_("detailed format of raw file"))
    parser.add_argument('--all', '-a', action="store_true", help=_("show logs for all days"))
    parser.add_argument('--nocolor', '-c', action="store_true", help=_("reset the output color"))
    parser.add_argument('--until', '-u', help=_("until some date (example: '01.01.2023' or '01.01.2023 12:00:00')"))
    parser.add_argument('--since', '-z', help=_("from some date (example: '01.01.2023' or '01.01.2023 12:00:00')"))
    parser.add_argument('--iso-date', '-q', action= "store_true", help = _('Convert time to ISO format'))
    parser.add_argument('--count', '-n', help=_('get last n events'))
    args = parser.parse_args()


    if not args.d:
        args.d = '0'
    if args.all:
        args.d = False
    if args.today:
        args.d = '0'

    return args


def data_output():
    try:
        args = get_args()
    except Exception as e:
        print(e)
        print('Check values of arguments')
        sys.exit(1)
    try:
        print(async_reading.start_reading(args))
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == '__main__':
    data_output()

