#!/usr/bin/python3


import gettext
import json
import re

import locale
import sys
from datetime import datetime

gettext.bindtextdomain('translate_logs',
                       '/usr/share/locale')
gettext.textdomain('translate_logs')
_ = gettext.gettext


class Logs:
    def __init__(self, new_line, way, source, priority, date, detailed, format, raw, nocolor, from_date, until_date,
                 time_format, count):
        self.locale_info = locale.getdefaultlocale()
        self.full_info_list = ['name_ru', 'time', 'name_en']
        self.short_content = ['DATE', 'PRIORITY']
        self.new_line = new_line
        self.way = way
        self.priority = priority
        self.source = source
        self.date = date
        self.detailed = detailed
        self.from_date = from_date
        self.until_date = until_date
        self.time_format = time_format
        self.format = format
        self.raw = raw
        self.nocolor = nocolor
        self.list_of_logs = self.new_line
        self.file_line = ''
        self.msg_info = ''
        self.line_break = ', '
        self.OKGREEN = '\033[93m'
        self.FAIL = '\033[91m'
        self.ENDC = '\033[0m'
        self.GREY = '\033[90m'
        self.count = count

        self.colors_en = {'critical': self.FAIL, 'crit': self.FAIL, 'alert': self.FAIL, 'emergency': self.FAIL,
                          'notice': self.ENDC,
                          'info': self.ENDC,
                          'debug': self.ENDC, 'warning': self.OKGREEN, 'error': self.OKGREEN, 'err': self.OKGREEN,
                          'emerg': self.FAIL}

    def open_file(self):
        self.list_of_logs = self.new_line
        return self.list_of_logs

    def parse_file(self):
        self.open_file()
        for dict_ in self.list_of_logs:
            try:
                self.file_line = json.loads(dict_)

                self.msg_info = self.file_line['MSG']
                self.file_line.pop('MSG')

                detailed_info = dict(**dict(list(self.msg_info.values())[0]), **self.file_line)

                if 'ISODATE' in list(detailed_info.keys()):
                    detailed_info['DATE'] = detailed_info['ISODATE']
                else:
                    if 'time' not in dict(list(self.msg_info.values())[0]).keys():
                        if 'start_time' in dict(list(self.msg_info.values())[0]).keys():
                            detailed_info['DATE'] = datetime.fromtimestamp(int(detailed_info['start_time'])).isoformat()
                            del detailed_info['start_time']
                    else:
                        detailed_info['DATE'] = detailed_info['time']
                        del detailed_info['time']

                source = list(self.msg_info.keys())[0].partition('-')[2]

                if 'type_en' not in list(detailed_info.keys()):
                    detailed_info['type_en'] = source
                    detailed_info['type_ru'] = source

                if self.locale_info[0] == 'ru_RU':
                    detailed_info.pop('name_en')
                    detailed_info.pop('type_en')
                else:
                    detailed_info.pop('name_ru')
                    detailed_info.pop('type_ru')

                short_logs_content = [self.file_line.get(key) for key in self.short_content][::-1]

                detailed_info['source'] = source

                short_content = self.msg_info[list(self.msg_info.keys())[0]]

                if 'time' not in list(short_content.keys()):
                    short_content["time"] = detailed_info["DATE"]

                if self.locale_info[0] == 'ru_RU':
                    event_logs = [short_content.get(key) for key in self.full_info_list[:-1]][::-1]
                else:
                    event_logs = [short_content.get(key) for key in self.full_info_list[1:]]
                if self.locale_info[0] == 'ru_RU':
                    event_logs.append(detailed_info['type_ru'])
                else:
                    event_logs.append(detailed_info['type_en'])
                event_logs = event_logs + short_logs_content
                event_logs[-1] = event_logs[0]
                event_logs.pop(0)

                list_of_logs_values = list([])
                list_of_logs_values.append(event_logs[::-1])
                self.translate_logs_to_locale(list_of_logs_values, detailed_info)
            except KeyboardInterrupt:
                sys.exit(1)
            except json.JSONDecodeError:
                pass
        return ''

    def translate_logs_to_locale(self, logs, detailed_info):
        logs_row = (el for el in logs)
        try:
            translated_logs = [_(el) for el in next(logs_row)[:7] if el != '']
            try:
                datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S')
            except:
                translated_logs[0] = str(datetime.fromisoformat(translated_logs[0]).strftime('%d.%m.%Y %H:%M:%S'))
            if self.until_date or self.from_date:
                if self.until_date and self.from_date:
                    if ':' in self.from_date and ':' in self.until_date:
                        log_datetime = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S')
                        from_date = datetime.strptime(self.from_date, '%d.%m.%Y %H:%M:%S')
                        until_date = datetime.strptime(self.until_date, '%d.%m.%Y %H:%M:%S')
                    else:
                        log_datetime = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S').date()
                        from_date = datetime.strptime(self.from_date, '%d.%m.%Y').date()
                        until_date = datetime.strptime(self.until_date, '%d.%m.%Y').date()

                    if log_datetime >= from_date and log_datetime <= until_date:
                        if self.time_format:
                            translated_logs[0] = datetime.strptime(f'{translated_logs[0]}',
                                                                   '%d.%m.%Y %H:%M:%S').isoformat()
                        self.check_functions(detailed_info, translated_logs)
                elif self.until_date:
                    if ':' in self.until_date:
                        log_datetime = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S')
                        until_date = datetime.strptime(self.until_date, '%d.%m.%Y %H:%M:%S')
                    else:
                        log_datetime = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S').date()
                        until_date = datetime.strptime(self.until_date, '%d.%m.%Y').date()
                    if log_datetime <= until_date:
                        if self.time_format:
                            translated_logs[0] = datetime.strptime(f'{translated_logs[0]}',
                                                                   '%d.%m.%Y %H:%M:%S').isoformat()
                        self.check_functions(detailed_info, translated_logs)
                elif self.from_date:
                    if ':' in self.from_date:
                        log_datetime = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S')
                        from_date = datetime.strptime(self.from_date, '%d.%m.%Y %H:%M:%S')
                    else:
                        log_datetime = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S').date()
                        from_date = datetime.strptime(self.from_date, '%d.%m.%Y').date()
                    if log_datetime >= from_date:
                        if self.time_format:
                            translated_logs[0] = datetime.strptime(f'{translated_logs[0]}',
                                                                   '%d.%m.%Y %H:%M:%S').isoformat()
                        self.check_functions(detailed_info, translated_logs)
            else:
                if self.time_format:
                    translated_logs[0] = datetime.strptime(f'{translated_logs[0]}', '%d.%m.%Y %H:%M:%S').isoformat()
                self.check_functions(detailed_info, translated_logs)

        except KeyboardInterrupt:
            sys.exit()
        except Exception as e:
            pass

    def check_functions(self, detailed_info, translated_logs):
        if self.format:
            self.set_format(detailed_info)
        elif self.detailed:
            detailed_info = {'DATE': detailed_info['DATE'], 'PRIORITY': detailed_info['PRIORITY'],
                             'message_id': detailed_info['message_id'], **detailed_info}  # replace positions
            if not self.priority:
                self.priority = ['']
            if not self.source:
                self.source = ['']
            for source in self.source:
                detailed_info['source'] = _(detailed_info['source'])
                detailed_info['PRIORITY'] = _(detailed_info['PRIORITY'])
                for priority in self.priority:
                    if source in detailed_info['source'] and priority in detailed_info['PRIORITY']:
                        print(self.detailed_output(detailed_info))

        elif self.raw:
            detailed_info = self.filter_for_detailed(detailed_info)
            print(self.detailed_raw(detailed_info))

        else:
            if not self.nocolor:
                translated_logs = self.colorize_row(translated_logs)
            if self.priority or self.source:
                translated_logs = self.args_filter(translated_logs)
                print(f'{self.line_break}'.join(translated_logs))
            else:
                print(f'{self.line_break}'.join(translated_logs))

    def detailed_output(self, detailed_info):
        max_key_len_ru = len(max([_(key.capitalize()) for key in list(detailed_info.keys())], key=len))

        max_key_len = len(max(detailed_info.keys(), key=len))
        if self.time_format:
            detailed_info['DATE'] = datetime.strptime(f'{detailed_info["DATE"]}', '%d.%m.%Y %H:%M:%S').isoformat()
        for k, v in detailed_info.items():
            if self.locale_info[0] == 'ru_RU':
                if not self.nocolor:
                    value = _(v)
                    print(
                        f'{self.GREY + _(k.capitalize()) + self.ENDC}{self.GREY + ":" + self.ENDC}{" " * (max_key_len_ru - len(_(k.capitalize())) + 1)}{value}')
                else:
                    print(f'{_(k.capitalize())}:{" " * (max_key_len_ru - len(_(k.capitalize())) + 1)}{v}')
            else:
                if not self.nocolor:
                    try:
                        value = self.colors_en[v] + v + self.ENDC
                    except:
                        value = v
                    print(
                        f'{self.GREY + k.capitalize() + self.ENDC}{self.GREY + ":" + self.ENDC}{" " * (max_key_len - len(k) + 1)}{value}')
                else:
                    print(f'{k.capitalize()}:{" " * (max_key_len - len(k) + 1)}{v}')
        return ''

    def set_format(self, detailed_info):
        format = self.format.split()
        for txt in format:
            if ' '.join(re.findall('%(.?.*)', txt)).replace(',', '') not in list(detailed_info.keys()):
                detailed_info[' '.join(re.findall('%(.?.*)', txt)).replace(',', '')] = '-'

        print(' '.join(
            [detailed_info[re.findall('%(.?.*)', txt)[0].replace(',', '')] if re.findall('%(.?.*)', txt) else txt for
             txt in format]))

    @staticmethod
    def detailed_raw(detailed_info):
        max_key_len = len(max(detailed_info.keys(), key=len))
        for k, v in detailed_info.items():
            print(f'{k}:{" " * (max_key_len - len(k) + 1)}{v}')
        return ''

    def colorize_row(self, translated_logs):
        translated_logs = [self.GREY + el + self.ENDC if
                           translated_logs.index(el) != len(translated_logs) - 1 and translated_logs.index(
                               el) != 1 else el for el in translated_logs]
        self.line_break = self.GREY + ', ' + self.ENDC
        translated_logs[1] = _(translated_logs[1])
        if self.locale_info[0] != 'ru_RU':
            translated_logs[1] = self.colors_en[translated_logs[1]] + translated_logs[1] + self.ENDC
        return translated_logs

    def args_filter(self, translated_logs):
        if not self.priority:
            self.priority = ['']
        else:
            self.priority = [_(priority) for priority in self.priority]
        if not self.source:
            self.source = ['']
        else:
            self.source = [_(source) for source in self.source]
        for source in self.source:
            for priority in self.priority:
                if source in translated_logs[2] and priority in translated_logs[1]:
                    return translated_logs

    def filter_for_detailed(self, detailed_info):
        if not self.priority:
            self.priority = ['']
        if not self.source:
            self.source = ['']
        for source in self.source:
            for priority in self.priority:
                if source in detailed_info['source'] and priority in detailed_info['PRIORITY']:
                    return detailed_info
